<?php
/**
 * Plugin Name: Autoblogging.ai Importer
 * Description: Autoblogging.ai Importer to Import articles from the CSV file generated on Autoblogging.ai and create WordPress posts, pages, or custom post types.
 * Version: 1.0
 * Author: Autoblogging.ai
 * Author URI: https://autoblogging.ai/
 * Text Domain: Autoblogging.ai
 */


require_once plugin_dir_path(__FILE__) . 'vendor/parsecsv/parsecsv.lib.php';
use ParseCsv\Csv;

register_activation_hook(__FILE__, 'autoblogging_ai_importer_activate');
function autoblogging_ai_importer_activate() {
    if (!wp_next_scheduled('autoblogging_ai_importer_cron_hook')) {
        wp_schedule_event(time(), 'hourly', 'autoblogging_ai_importer_cron_hook');
    }
}

register_deactivation_hook(__FILE__, 'autoblogging_ai_importer_deactivate');
function autoblogging_ai_importer_deactivate() {
    $timestamp = wp_next_scheduled('autoblogging_ai_importer_cron_hook');
    wp_unschedule_event($timestamp, 'autoblogging_ai_importer_cron_hook');
}

add_action('autoblogging_ai_importer_cron_hook', 'autoblogging_ai_process_batch_update');
add_action('admin_menu', 'csv_to_wp_posts_menu');
add_action('admin_post_csv_to_wp_posts_import', 'handle_csv_import');
add_action('admin_enqueue_scripts', 'csv_to_wp_posts_enqueue_scripts');
add_action('save_post', 'process_autoblogging_images');
add_filter('wp_kses_allowed_html', 'allow_script_tags', 10, 2);

function csv_to_wp_posts_menu() {
    add_menu_page('CSV to WordPress Posts', 'Autoblogging.ai Importer', 'manage_options', 'csv-to-wp-posts', 'csv_to_wp_posts_page');
    add_submenu_page('csv-to-wp-posts', 'Task Status', 'Status of Pending Tasks', 'manage_options', 'csv-to-wp-posts-status', 'autoblogging_ai_importer_status_page');
}

function csv_to_wp_posts_page() {
    $categories = get_categories(array('hide_empty' => false));
    echo '<div class="wrap">';
    echo '<h1>Autoblogging.ai\'s Bulk Importer</h1>';
    echo '<form action="' . admin_url('admin-post.php') . '" method="post" enctype="multipart/form-data">';
    echo '<input type="hidden" name="action" value="csv_to_wp_posts_import">';
    echo '<input type="file" name="csv_file" accept=".csv" required>';
    echo '<select name="post_type" id="post_type">';
    echo '<option value="post">Post</option>';
    echo '<option value="page">Page</option>';
    echo '</select>';
    echo '<div id="post_categories" style="display:none;">';
    display_categories_checkboxes($categories);
    echo '</div>';
    echo '<select name="post_status" id="post_status">';
    echo '<option value="publish">Publish</option>';
    echo '<option value="draft">Draft</option>';
    echo '</select>';
    echo '<input type="submit" value="Import CSV">';
    echo '</form>';
    echo '</div>';
}

function display_categories_checkboxes($categories, $parent_id = 0, $level = 0) {
    foreach ($categories as $category) {
        if ($category->parent == $parent_id) {
            echo str_repeat("&nbsp;&nbsp;", $level);
            echo '<input type="checkbox" name="post_categories[]" value="' . esc_attr($category->term_id) . '"> ';
            echo esc_html($category->name) . '<br>';
            display_categories_checkboxes($categories, $category->term_id, $level + 1);
        }
    }
}

function handle_csv_import() {
    if (!isset($_FILES['csv_file']) || $_FILES['csv_file']['error'] != UPLOAD_ERR_OK) {
        wp_redirect(admin_url('admin.php?page=csv-to-wp-posts&error=1'));
        exit;
    }
    $csv = new Csv();
    $csv->auto($_FILES['csv_file']['tmp_name']);
    $postType = sanitize_text_field($_POST['post_type']);
    $postStatus = sanitize_text_field($_POST['post_status']);
    $postCategories = isset($_POST['post_categories']) ? $_POST['post_categories'] : [];
    foreach ($csv->data as $record) {
        $batch_id = 'autoblogging_ai_' . md5(uniqid('', true));
        set_transient($batch_id, ['batch' => [$record], 'type' => $postType, 'status' => 'pending', 'timestamp' => current_time('mysql'), 'post_status' => $postStatus, 'post_categories' => $postCategories], 60 * 60);
        $tasks = get_option('autoblogging_ai_tasks', []);
        $tasks[$batch_id] = ['status' => 'pending', 'timestamp' => current_time('mysql')];
        update_option('autoblogging_ai_tasks', $tasks);
        wp_schedule_single_event(time(), 'autoblogging_ai_importer_cron_hook', [$batch_id]);
    }
    wp_redirect(admin_url('admin.php?page=csv-to-wp-posts&success=1'));
    exit;
}

function autoblogging_ai_process_batch_update($batch_id) {
    $batch_data = get_transient($batch_id);
    if ($batch_data) {
        $batch = $batch_data['batch'];
        $postType = $batch_data['type'];
        $postStatus = $batch_data['post_status'];
        $postCategories = $batch_data['post_categories'];
        foreach ($batch as $data) {
            $articleContent = trim($data['Article Content']);
            $allowed_tags = wp_kses_allowed_html('post');
            $allowed_tags['script'] = array('type' => true, 'src' => true, 'async' => true, 'defer' => true);
            $articleContent = wp_kses($articleContent, $allowed_tags);
            $post_data = [
                'post_title' => sanitize_text_field($data['Article Title']),
                'post_content' => wp_kses_post($articleContent),
                'post_status' => $postStatus,
                'post_type' => $postType,
            ];
            $postId = wp_insert_post($post_data);
            if ($postId && !empty($postCategories)) {
                wp_set_post_terms($postId, $postCategories, 'category');
            }
        }
        $tasks = get_option('autoblogging_ai_tasks', []);
        $tasks[$batch_id]['status'] = 'completed';
        $tasks[$batch_id]['completed_timestamp'] = current_time('mysql');
        update_option('autoblogging_ai_tasks', $tasks);
        delete_transient($batch_id);
    }
}

function autoblogging_ai_importer_status_page() {
    $tasks = get_option('autoblogging_ai_tasks', []);
    echo '<h1>Status of Pending Tasks</h1>';
    echo '<table class="widefat">';
    echo '<thead>';
    echo '<tr><th>Batch ID</th><th>Status</th><th>Timestamp</th><th>Completed Timestamp</th></tr>';
    echo '</thead>';
    echo '<tbody>';
    foreach ($tasks as $batch_id => $task) {
        echo '<tr>';
        echo '<td>' . esc_html($batch_id) . '</td>';
        echo '<td>' . esc_html($task['status']) . '</td>';
        echo '<td>' . esc_html($task['timestamp']) . '</td>';
        echo '<td>' . esc_html($task['completed_timestamp'] ?? 'N/A') . '</td>';
        echo '</tr>';
    }
    echo '</tbody>';
    echo '</table>';
}

function csv_to_wp_posts_enqueue_scripts($hook) {
    if ($hook != 'toplevel_page_csv-to-wp-posts') {
        return;
    }
    wp_enqueue_script('csv-to-wp-posts-js', plugins_url('/js/csv-to-wp-posts.js', __FILE__), ['jquery'], null, true);
    wp_enqueue_style('csv-to-wp-posts-css', plugins_url('/css/csv-to-wp-posts.css', __FILE__));
}

function allow_script_tags($allowed_tags) {
    $allowed_tags['script'] = array('type' => true, 'src' => true, 'async' => true, 'defer' => true);
    return $allowed_tags;
}

function process_autoblogging_images($post_id) {
    if (wp_is_post_revision($post_id)) {
        return;
    }
    remove_action('save_post', 'process_autoblogging_images');
    $post = get_post($post_id);
    $post_content = $post->post_content;
    preg_match_all('/<img\s[^>]*?src\s*=\s*([\'"])(https?:\/\/files\.autoblogging\.ai[^\'"]+)\1[^>]*?>/i', $post_content, $matches, PREG_SET_ORDER);
    $featured_image_set = has_post_thumbnail($post_id);
    $upload_dir = wp_upload_dir();
    $post_title = sanitize_title($post->post_title);
    foreach ($matches as $match) {
        $image_url = $match[2];
        $image_data = file_get_contents($image_url);
        $filename = $post_title . '-' . wp_generate_password(2, false) . '.' . pathinfo($image_url, PATHINFO_EXTENSION);
        $filename = wp_unique_filename($upload_dir['path'], $filename);
        $file = $upload_dir['path'] . '/' . $filename;
        file_put_contents($file, $image_data);
        $alt_text = '';
        preg_match('/alt=[\'"](.*?)[\'"]/i', $match[0], $alt_matches);
        if (isset($alt_matches[1])) {
            $alt_text = $alt_matches[1];
        }
        $attachment = array(
            'guid' => $upload_dir['url'] . '/' . $filename,
            'post_mime_type' => 'image/jpeg',
            'post_title' => $alt_text,
            'post_content' => '',
            'post_status' => 'inherit',
            'post_excerpt' => $alt_text,
        );
        $attachment_id = wp_insert_attachment($attachment, $file, $post_id);
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $attach_data = wp_generate_attachment_metadata($attachment_id, $file);
        wp_update_attachment_metadata($attachment_id, $attach_data);
        if (!$featured_image_set) {
            set_post_thumbnail($post_id, $attachment_id);
            $featured_image_set = true;
        }
        $new_url = wp_get_attachment_url($attachment_id);
        $post_content = str_replace($match[2], $new_url, $post_content);
    }
    wp_update_post(array(
        'ID' => $post_id,
        'post_content' => $post_content,
    ));
    add_action('save_post', 'process_autoblogging_images');
}
